#!/bin/bash

if [ $(id -u) -ne 0 ] ; then echo "Must be run as root"; exit 1; fi

check_exists() { if ! hash $1 2>/dev/null ; then echo $1 not found ; exit 1 ; fi ; }


check_exists dd
check_exists sfdisk
check_exists mkfs.ext4
check_exists id
check_exists mkfs.f2fs
check_exists lsblk
check_exists pv
check_exists partprobe
check_exists blkid

if [ "$#" -ne 3 ] ; then echo "Usage: $0 : rootfs.img.tar.zst u-boot /dev/mmcblk1" ; exit 1 ; fi

IMAGE=$1
UBOOT=$2
DISK=$3
ROOTFS=$(mktemp -d)



if [ ! -f "${UBOOT}/MLO" ] || [ ! -f "${UBOOT}/u-boot.img" ] ; then echo "u-boot files not found"; exit 1; fi
if [ ! -f $IMAGE ] ; then echo "rootfs image $IMAGE not found."; exit 1; fi

echo "Installing $IMAGE to $DISK , make sure all partitions are unmounted"
read -p "Are you sure you want to continue? <y/N> " prompt
if ! [[ $prompt =~ [yY](es)* ]] ; then echo "Exiting.." ; exit 1 ; fi

if [[ $DISK == /dev/mmcblk* ]] ; then
	PART_ROOTFS=${DISK}p2
    PART_BOOT=${DISK}p1

else
	PART_ROOTFS=${DISK}2
    PART_BOOT=${DISK}1
fi

dd if=/dev/zero of=${DISK} bs=1M count=10
partprobe ${DISK}
dd if=${UBOOT}/MLO of=/dev/mmcblk0 count=1 seek=2 bs=128k
dd if=${UBOOT}/u-boot.img of=/dev/mmcblk0 count=2 seek=1 bs=384k
echo -e "1M,256M,0x83,* \n ,,0x83," | sfdisk $DISK
partprobe ${DISK}
mke2fs -q -L boot ${PART_BOOT}
mkfs.f2fs -q -f -l rootfs ${PART_ROOTFS}
sync
mkdir -p ${ROOTFS}
mount ${PART_ROOTFS} ${ROOTFS}
mkdir -p ${ROOTFS}/boot
mount ${PART_BOOT} ${ROOTFS}/boot

echo "
${UBOOT}


Writing ${IMAGE} to $DISK"
pv -w 106 $IMAGE | zstd -d - | tar --numeric-owner -xf - -C ${ROOTFS}
echo "Syncing...."
sync

. ${ROOTFS}/etc/default/pyra-extlinux
OLDROOT=$( echo ${EXTLINUX_ROOT} | sed 's/root=//')
ROOTPARTUUID=$(blkid -p $PART_ROOTFS -s PART_ENTRY_UUID -o value)
BOOTPARTUUID=$(blkid -p $PART_BOOT -s PART_ENTRY_UUID -o value)

cat > ${ROOTFS}/etc/fstab << EOF
# /etc/fstab: static file system information.
#
# Use 'blkid' to print the universally uniqu+e identifier for a
# device; this may be used with UUID= as a more robust way to name devices
# that works even if disks are added and removed. See fstab(5).
#
# <file system> <mount point>   <type>  <options>       <dump>  <pass>
PARTUUID=${ROOTPARTUUID}  /               f2fs    defaults        0       1
PARTUUID=${BOOTPARTUUID}  /boot           ext4    defaults        0       1

EOF

sed -i "s/${OLDROOT}/PARTUUID=${ROOTPARTUUID}/" ${ROOTFS}/boot/extlinux/linux.cfg
sed -i "s/${OLDROOT}/PARTUUID=${ROOTPARTUUID}/" ${ROOTFS}/etc/default/pyra-extlinux
sync


umount ${PART_BOOT}
umount ${PART_ROOTFS}
rm -rf ${ROOTFS}

echo "Done"
